/*
 *  ttyvaders     Textmode shoot'em up
 *  Copyright (c) 2002 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This program is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "config.h"

#include <stdlib.h>

#include "common.h"

caca_canvas_t *foo_sprite;
caca_canvas_t *bar_sprite;
caca_canvas_t *baz_sprite;

void init_aliens(game *g, aliens *al)
{
    int i;

    for(i = 0; i < ALIENS; i++)
    {
        al->type[i] = ALIEN_NONE;
    }

    foo_sprite = caca_create_canvas(0, 0);
    caca_import_file(foo_sprite, "data/foofight.txt", "utf8");

    bar_sprite = caca_create_canvas(0, 0);
    caca_import_file(bar_sprite, "data/barfight.txt", "utf8");

    baz_sprite = caca_create_canvas(0, 0);
    caca_import_file(baz_sprite, "data/bazfight.txt", "utf8");
}

void draw_aliens(game *g, aliens *al)
{
    int i;

    for(i = 0; i < ALIENS; i++)
    {
        switch(al->type[i])
        {
            case ALIEN_FOO:
                caca_set_frame(foo_sprite, al->img[i] % 5);
                caca_blit(g->cv, al->x[i], al->y[i], foo_sprite, NULL);
                break;
            case ALIEN_BAR:
                caca_set_frame(bar_sprite, al->img[i] % 2);
                caca_blit(g->cv, al->x[i], al->y[i], bar_sprite, NULL);
                break;
            case ALIEN_BAZ:
                caca_set_frame(baz_sprite, al->img[i] % 4);
                caca_blit(g->cv, al->x[i], al->y[i], baz_sprite, NULL);
                break;
            case ALIEN_NONE:
                break;
        }
    }
}

void update_aliens(game *g, aliens *al)
{
    int i;

    for(i = 0; i < ALIENS; i++)
    {
        /* If alien died, make it explode */
        if(al->type[i] != ALIEN_NONE && al->life[i] < 0)
        {
            add_explosion(g, g->ex, al->x[i], al->y[i], 0, 0, EXPLOSION_MEDIUM);
            al->type[i] = ALIEN_NONE;
            add_bonus(g, g->bo, al->x[i], al->y[i], caca_rand(0,4) ? BONUS_GREEN : BONUS_LIFE);
        }

        /* Update coordinates */
        switch(al->type[i])
        {
            case ALIEN_FOO:
            case ALIEN_BAR:
            case ALIEN_BAZ:
                al->x[i] = ((al->x[i] + 5) % (g->w + 3)) - 3;
                al->y[i] = al->y[i] + (rand() % 8) / 7 - (rand() % 8) / 7;
                al->img[i] = al->img[i] + 1;

                /* Check bounds */
                if(al->y[i] < 0 ) al->y[i] = 0;
                if(al->y[i] > g->w - 1 ) al->y[i] = g->w - 1;
                break;
            case ALIEN_NONE:
                break;
        }
    }
}

void add_alien(game *g, aliens *al, int x, int y, int type)
{
    int i;

    for(i = 0; i < ALIENS; i++)
    {
        if(al->type[i] == ALIEN_NONE)
        {
            al->type[i] = type;
            al->x[i] = x;
            al->y[i] = y;
            al->img[i] = 0;

            switch(al->type[i])
            {
                case ALIEN_FOO:
                    al->life[i] = 3;
                    break;
                case ALIEN_BAR:
                    al->life[i] = 3;
                    break;
                case ALIEN_BAZ:
                    al->life[i] = 3;
                    break;
                case ALIEN_NONE:
                    break;
            }

            break;
        }
    }
}

