/*
 *  ttyvaders     Textmode shoot'em up
 *  Copyright (c) 2002-2003 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This program is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "config.h"

#include <stdlib.h>

#include "common.h"

box * create_box(game *g, int x, int y, int w, int h)
{
    box *b = malloc(sizeof(box));
    if(b == NULL)
        exit(1);

    b->x = x;
    b->y = y;
    b->w = w;
    b->h = h;
    b->frame = 0;

    return b;
}

void draw_box(game *g, box *b)
{
    int j, frame;

    caca_set_color_ansi(g->cv, CACA_YELLOW, CACA_BLACK);

    /* Draw the thin horizontal line */
    if(b->frame < 8)
    {
        caca_draw_line(g->cv, b->x - b->w * b->frame / 16, b->y,
                               b->x + b->w * b->frame / 16 - 1, b->y, 'X');
        return;
    }

    /* Draw the frame */
    frame = b->frame < 12 ? b->frame : 12;

    caca_draw_line(g->cv, b->x - b->w / 2, b->y - b->h * (frame - 8) / 8,
                    b->x + b->w / 2 - 1, b->y - b->h * (frame - 8) / 8, 'X');
    caca_draw_line(g->cv, b->x - b->w / 2, b->y + b->h * (frame - 8) / 8,
                    b->x + b->w / 2 - 1, b->y + b->h * (frame - 8) / 8, 'X');

    caca_draw_line(g->cv, b->x - b->w / 2, b->y - b->h * (frame - 8) / 8,
                    b->x - b->w / 2, b->y + b->h * (frame - 8) / 8 - 1, 'X');
    caca_draw_line(g->cv, b->x + b->w / 2 - 1, b->y - b->h * (frame - 8) / 8,
               b->x + b->w / 2 - 1, b->y + b->h * (frame - 8) / 8 - 1, 'X');

    caca_set_color_ansi(g->cv, CACA_BLACK, CACA_BLACK);

    for(j = b->y - b->h * (frame - 8) / 8 + 1;
         j < b->y + b->h * (frame - 8) / 8;
         j++)
    {
        caca_draw_line(g->cv, b->x - b->w / 2 + 1, j,
                        b->x + b->w / 2 - 2, j, 'X');
    }

    if(b->frame < 12)
    {
        return;
    }

    /* Draw the text inside the frame */
    caca_set_color_ansi(g->cv, CACA_YELLOW, CACA_BLACK);

    /* FIXME: use a font */
    caca_put_str(g->cv, b->x - b->w / 2 + 12, b->y - b->h / 2 + 2,
                 "XXXX.  .XXXX  X   X  .XXXX  .XXXX  XXXX.");
    caca_put_str(g->cv, b->x - b->w / 2 + 12, b->y - b->h / 2 + 3,
                 "X  `X  X'  X  X   X  X'     X'     X  `X");
    caca_put_str(g->cv, b->x - b->w / 2 + 12, b->y - b->h / 2 + 4,
                 "XXXX'  XXXXX  X   X  `XXX   XXXX   X   X");
    caca_put_str(g->cv, b->x - b->w / 2 + 12, b->y - b->h / 2 + 5,
                 "X'     X' `X  X. ,X     `X  X'     X  ,X");
    caca_put_str(g->cv, b->x - b->w / 2 + 12, b->y - b->h / 2 + 6,
                 "X      X   X  `XXXX  XXXX'  `XXXX  XXXX'");
}

void free_box(box *b)
{
    free(b);
}

