/*
 *  ttyvaders     Textmode shoot'em up
 *  Copyright (c) 2002 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This program is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "config.h"

#include <stdlib.h>

#include "common.h"

caca_canvas_t *medium_sprite;
caca_canvas_t *small_sprite;

void init_explosions(game *g, explosions *ex)
{
    int i;

    for(i = 0; i < EXPLOSIONS; i++)
    {
        ex->type[i] = EXPLOSION_NONE;
    }

    medium_sprite = caca_create_canvas(0, 0);
    caca_import_file(medium_sprite, "data/xplmed.txt", "utf8");

    small_sprite = caca_create_canvas(0, 0);
    caca_import_file(small_sprite, "data/xplsmall.txt", "utf8");
}

void add_explosion(game *g, explosions *ex, int x, int y, int vx, int vy, int type)
{
    int i;

    for(i = 0; i < EXPLOSIONS; i++)
    {
        if(ex->type[i] == EXPLOSION_NONE)
        {
            ex->type[i] = type;
            ex->x[i] = x;
            ex->y[i] = y;
            ex->vx[i] = vx;
            ex->vy[i] = vy;
            switch(type)
            {
                case EXPLOSION_MEDIUM:
                    ex->n[i] = 11;
                    break;
                case EXPLOSION_SMALL:
                    ex->n[i] = 7;
                    break;
            }
            break;
        }
    }
}

void draw_explosions(game *g, explosions *ex)
{
    int i;

    for(i = 0; i < EXPLOSIONS; i++)
    {
#if 0
        caca_set_fg_color(g->cv, CACA_GREEN);
        caca_goto(g->cv, ex->x[i] + 3, ex->y[i]);
        switch(caca_rand(0,2))
        {
        case 0:
            caca_putchar(g->cv, 'p');
            caca_putchar(g->cv, 'i');
            caca_putchar(g->cv, 'f');
            break;
        case 1:
            caca_putchar(g->cv, 'p');
            caca_putchar(g->cv, 'a');
            caca_putchar(g->cv, 'f');
            break;
        case 2:
            caca_putchar(g->cv, 'p');
            caca_putchar(g->cv, 'o');
            caca_putchar(g->cv, 'u');
            caca_putchar(g->cv, 'f');
            break;
        }
        caca_putchar(g->cv, '!');
#endif

        switch(ex->type[i])
        {
            case EXPLOSION_MEDIUM:
                caca_set_frame(medium_sprite, 10 - ex->n[i]);
                caca_blit(g->cv, ex->x[i], ex->y[i], medium_sprite, NULL);
                break;
            case EXPLOSION_SMALL:
                caca_set_frame(small_sprite, 6 - ex->n[i]);
                caca_blit(g->cv, ex->x[i], ex->y[i], small_sprite, NULL);
                break;
            case EXPLOSION_NONE:
                break;
        }
    }
}

void update_explosions(game *g, explosions *ex)
{
    int i;

    for(i = 0; i < EXPLOSIONS; i++)
    {
        switch(ex->type[i])
        {
            case EXPLOSION_MEDIUM:
            case EXPLOSION_SMALL:
                ex->x[i] += ex->vx[i];
                ex->y[i] += ex->vy[i];
                ex->n[i]--;
                if(ex->n[i] < 0)
                {
                    ex->type[i] = EXPLOSION_NONE;
                }
                break;
            case EXPLOSION_NONE:
                break;
        }
    }
}

